﻿using Hotline.Application.Snapshot;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Snapshot;
using Microsoft.AspNetCore.Mvc;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Tools;
using Hotline.Snapshot.Interfaces;
using Hotline.Caching.Interfaces;

namespace Hotline.Api.Controllers.Snapshot;

public class RedPackController : BaseController
{
    private readonly IRedPackApplication _redPackApplication;
    private readonly IIndustryRepository _industryRepository;
    private readonly ISystemDicDataCacheManager _sysDic;

    public RedPackController(IRedPackApplication redPackApplication, IIndustryRepository industryRepository, ISystemDicDataCacheManager sysDic)
    {
        _redPackApplication = redPackApplication;
        _industryRepository = industryRepository;
        _sysDic = sysDic;
    }

    /// <summary>
    /// 获取市民红包审批列表
    /// </summary>
    /// <returns></returns>
    [HttpGet("audit")]
    public async Task<PagedDto<SnapshotOrderAuditItemsOutDto>> GetRedPackAuditItemsAsync([FromQuery] SnapshotOrderAuditItemsInDto dto)
        => (await _redPackApplication.GetRedPackAuditItemsAsync(dto).ToPagedListAsync(dto)).ToPaged();

    [HttpGet("audit/basedata")]
    public async Task<Dictionary<string, object>> GetAuditBaseDataAsync()
    {
        var industry = await _industryRepository.Queryable()
            .Select(m => new { m.Id, m.Name })
            .ToListAsync();
        return new Dictionary<string, object>
        {
            { "industry", industry },
            { "configAmount", _sysDic.IndustryConfigAmount }
        };
    }
}
