﻿using AutoFixture;
using Hotline.Api.Controllers;
using Hotline.Api.Controllers.Snapshot;
using Hotline.Application.Tests.Dto;
using Hotline.Orders;
using Hotline.Repository.SqlSugar.Orders;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Dtos.Snapshot;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Snapshot;
using Hotline.Share.Tools;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Authentications;
using XF.Domain.Repository;

namespace Hotline.Application.Tests.Mock;
public class OrderServiceMock
{
    private readonly OrderController _orderController;
    private CreateOrderOutDto CreateOrderOutDto;
    private AddOrderDto AddOrderDto;
    private readonly IRepository<Order> _orderRepository;
    private readonly IOrderVisitRepository _orderVisitRepository;
    private readonly SnapshotController _snapshotController;
    public readonly IFixture _fixture;
    public readonly ISessionContext _sessionContext;

    public OrderServiceMock(OrderController orderController, IRepository<Order> orderRepository, IOrderVisitRepository orderVisitRepository, SnapshotController snapshotController, ISessionContext sessionContext)
    {
        _orderController = orderController;
        _orderController.ControllerContext = new ControllerContext
        {
            HttpContext = new DefaultHttpContext()
        };
        _orderRepository = orderRepository;
        _orderVisitRepository = orderVisitRepository;
        _snapshotController = snapshotController;
        _snapshotController.ControllerContext = new ControllerContext
        {
            HttpContext = new DefaultHttpContext()
        };
        _fixture = new Fixture();
        _sessionContext = sessionContext;
    }

    public OrderServiceMock CreateSnapshotOrder()
    {
        var homePage = _snapshotController.GetHomePageAsync().GetAwaiter().GetResult();
        var industry = homePage.Industrys
            .Where(m => m.IndustryType == EIndustryType.Clue)
            .FirstOrDefault();
        var pageBase = _snapshotController.GetIndustryBaseAsync(industry.Id).GetAwaiter().GetResult();

        var inDto = _fixture.Create<AddSnapshotOrderInDto>();
        inDto.Street = "单元测试街道" + DateTime.Now.ToLongDateTimeString();
        inDto.IndustryId = industry.Id;
        inDto.Town = "仙市镇";
        inDto.County = "沿滩区";
        inDto.Description = "单元测试添加的时间描述";
        inDto.IsSecret = false;
        inDto.PhoneNumber = _sessionContext.Phone;
        inDto.Latitude = 29.3661181959828;
        inDto.Longitude = 104.41928;
        //inDto.JobType = 
        foreach (var item in inDto.Files)
        {
            item.FileName = DateTime.Now.ToShortTimeString() + "文件.doc";
        }
        CreateOrderOutDto = _snapshotController.AddOrderAsync(inDto).GetAwaiter().GetResult().ToJson().FromJson<CreateOrderOutDto>();
        return this;
    }

    public OrderServiceMock CreateOrder(string callId = "")
    {
        var json = "{\"sourceChannel\":\"因特网\",\"sourceChannelCode\":\"YTW\",\"transferPhone\":null,\"fromPhone\":null,\"acceptorName\":\"单元测试\",\"acceptorStaffNo\":\"\",\"fromName\":\"1233333333\",\"fromGender\":1,\"identityType\":1,\"licenceType\":null,\"licenceTypeCode\":null,\"licenceNo\":null,\"ageRange\":null,\"ageRangeCode\":null,\"contact\":\"12333333333\",\"isSecret\":false,\"acceptSms\":false,\"no\":null,\"title\":\"\",\"hotspotId\":\"1912\",\"eventCategoryId\":null,\"incidentTime\":null,\"incidentPurpose\":null,\"areaCode\":\"519800\",\"city\":\"省内\",\"street\":null,\"isRepeat\":\"false\",\"pushType\":null,\"pushTypeCode\":null,\"content\":\"单元测试内容\",\"duplicateIds\":[],\"duplicateTitle\":null,\"callAddress\":null,\"repeatableEventDetails\":[],\"orderExtension\":null,\"transpond\":false,\"isEnforcementOrder\":false,\"focusOnEventsArr\":[],\"focusOnEvents\":null,\"isFormalistWorkOrder\":false,\"isSensitiveWorkOrders\":false,\"isUrgent\":false,\"isThreePartyConference\":false,\"is24HoursComplete\":false,\"company\":null,\"orderPushTypes\":[],\"acceptType\":\"咨询\",\"acceptTypeCode\":\"10\",\"files\":[],\"hotspotSpliceName\":\"互联互通-转接乐山市12345\",\"hotspotName\":\"转接乐山市12345\",\"hotspotCode\":\"1912\",\"hotspotExternal\":\"19\",\"county\":\"\",\"town\":\"\"}";
        AddOrderDto = json.FromJson<AddOrderDto>();
        if (callId.NotNullOrEmpty())
        {
            AddOrderDto.CallId = callId;
            AddOrderDto.SourceChannel = "电话";
            AddOrderDto.SourceChannelCode = AppDefaults.SourceChannel.DianHua;
        }
        AddOrderDto.Title = "单元测试" + DateTime.Now.ToString("yyyy-MM-dd hh:mm:ss");
        CreateOrderOutDto = _orderController.Add(AddOrderDto).GetAwaiter().GetResult().ToJson().FromJson<CreateOrderOutDto>();
        return this;
    }

    public CreateOrderOutDto GetCreateResult()
    {
        return CreateOrderOutDto;
    }

    public OrderServiceMock 办理到派单员(Action action = null)
    {
        action?.Invoke();
        var stepNextInfo = _orderController.GetFlowStartOptions(CreateOrderOutDto.Id).GetAwaiter().GetResult().ToJson().FromJson<NextStepsDto<NextStepOption>>();
        var stepInfo = stepNextInfo.Steps.FirstOrDefault(m => m.Value == "派单组");
        var stepOrg = stepInfo.Items.FirstOrDefault(m => m.Username == "单元测试派单员");

        var handleDto = new StartWorkflowDto<OrderHandleFlowDto>
        {
            Data = new OrderHandleFlowDto
            {
                OrderId = CreateOrderOutDto.Id,
            },
            Workflow = new BasicWorkflowDto
            {
                NextHandlers = [stepOrg],
                NextStepCode = stepInfo.Key,
                NextStepName = stepInfo.Value,
                Opinion = "办理到派单组意见",
                BackToCountersignEnd = false,
                IsSms = false,
                IsForwarded = false,
                HandlerType = EHandlerType.OrgLevel,
                BusinessType = EBusinessType.Send,
                FlowDirection = EFlowDirection.CenterToCenter,
            }
        };
        _orderController.StartFlow(handleDto).GetAwaiter().GetResult();
        return this;

    }

    public OrderServiceMock 办理到一级部门()
    {
        var stepNextInfo = _orderController.GetFlowStartOptions(CreateOrderOutDto.Id).GetAwaiter().GetResult().ToJson().FromJson<NextStepsDto<NextStepOption>>();
        var stepInfo = stepNextInfo.Steps.FirstOrDefault(m => m.Value == "一级部门");
        var stepOrg = stepInfo.Items.FirstOrDefault(m => m.OrgName == "测试部门");

        var handleDto = new StartWorkflowDto<OrderHandleFlowDto>
        {
            Data = new OrderHandleFlowDto
            {
                OrderId = CreateOrderOutDto.Id,
            },
            Workflow = new BasicWorkflowDto
            {
                NextHandlers = [stepOrg],
                NextStepCode = stepInfo.Key,
                NextStepName = stepInfo.Value,
                Opinion = "办理意见",
                BackToCountersignEnd = false,
                IsSms = false,
                IsForwarded = false,
                HandlerType = EHandlerType.OrgLevel,
                BusinessType = EBusinessType.Department,
            }
        };
        _orderController.StartFlow(handleDto).GetAwaiter().GetResult();
        return this;
    }

    public OrderServiceMock 办理到归档(Action action = null, Action<OrderHandleFlowDto> dataAction = null, Action<NextWorkflowDto> workflowAction = null)
    {
        action?.Invoke();
        var stepNextInfo = _orderController.GetNextStepsWithRecommend(CreateOrderOutDto.Id).GetAwaiter().GetResult().ToJson().FromJson<NextStepsDto<NextStepOption>>();
        var stepInfo = stepNextInfo.Steps.FirstOrDefault(m => m.Value == "归档");
        var stepOrg = stepInfo.Items.FirstOrDefault();

        var order = _orderRepository.Get(CreateOrderOutDto.Id);

        var handleDto = new NextWorkflowDto<OrderHandleFlowDto>
        {
            Data = new OrderHandleFlowDto
            {
                OrderId = CreateOrderOutDto.Id,
                RealHandlerName = "经办人",
                RealHandlerPhone = "13666666666",
                RealContactLocale = true,
                RealIsContacted = true,
                IsOther = true,
                OtherRemark = "其它原因",
                RealCommunicationAddress = "地点地点地点",
                RealCommunicationTime = DateTime.Now,
            },
            Workflow = new NextWorkflowDto
            {
                NextStepCode = "end",
                NextStepName = "归档",
                Opinion = "办理到归档",
                BackToCountersignEnd = false,
                IsSms = false,
                IsForwarded = false,
                HandlerType = EHandlerType.OrgLevel,
                BusinessType = EBusinessType.Department,
                FlowDirection = EFlowDirection.OrgToOrg,
                WorkflowId = order.WorkflowId,
                StepId = stepNextInfo.StepId,
            }
        };
        dataAction?.Invoke(handleDto.Data);
        workflowAction?.Invoke(handleDto.Workflow);
        try
        {
            _orderController.Handle(handleDto).GetAwaiter().GetResult();
        }
        catch (Exception e)
        {
            var msg = e.Message;
            if (msg.Contains("RealtimeService") == false)
            {
                throw;
            }
            // ignore
        }
        return this;
    }

    public OrderServiceMock 办理到二级部门(Action action = null)
    {
        action?.Invoke();
        var stepNextInfo = _orderController.GetNextStepsWithRecommend(CreateOrderOutDto.Id).GetAwaiter().GetResult().ToJson().FromJson<NextStepsDto<NextStepOption>>();
        var stepInfo = stepNextInfo.Steps.FirstOrDefault(m => m.Value == "二级部门");
        var stepOrg = stepInfo.Items.FirstOrDefault(m => m.OrgName == "测试二级部门");

        var order = _orderRepository.Get(CreateOrderOutDto.Id);

        var handleDto = new NextWorkflowDto<OrderHandleFlowDto>
        {
            Data = new OrderHandleFlowDto
            {
                OrderId = CreateOrderOutDto.Id,
            },
            Workflow = new NextWorkflowDto
            {
                NextHandlers = [stepOrg],
                NextStepCode = stepInfo.Key,
                NextStepName = stepInfo.Value,
                Opinion = "办理到二级部门",
                BackToCountersignEnd = false,
                IsSms = false,
                IsForwarded = false,
                HandlerType = EHandlerType.OrgLevel,
                BusinessType = EBusinessType.Department,
                FlowDirection = EFlowDirection.OrgToOrg,
                WorkflowId = order.WorkflowId,
                StepId = stepNextInfo.StepId,
            }
        };
        _orderController.Handle(handleDto).GetAwaiter().GetResult();
        return this;
    }

    public OrderServiceMock 办理一级部门汇总(Action aciton = null)
    {
        aciton?.Invoke();
        var stepNextInfo = _orderController.GetNextStepsWithRecommend(CreateOrderOutDto.Id).GetAwaiter().GetResult().ToJson().FromJson<NextStepsDto<NextStepOption>>();
        var stepInfo = stepNextInfo.Steps.FirstOrDefault(m => m.Value == "一级部门汇总");
        var stepOrg = stepInfo.Items.FirstOrDefault(m => m.OrgName == "测试部门");

        var order = _orderRepository.Get(CreateOrderOutDto.Id);

        var handleDto = new NextWorkflowDto<OrderHandleFlowDto>
        {
            Data = new OrderHandleFlowDto
            {
                OrderId = CreateOrderOutDto.Id,
                RealHandlerName = "经办人",
                RealHandlerPhone = "13666666666",
                RealContactLocale = true,
                RealIsContacted = true,
                IsOther = true,
                IsEvasive = true,
                OtherRemark = "其它原因",
                RealCommunicationAddress = "地点地点地点",
                RealCommunicationTime = DateTime.Now
            },
            Workflow = new NextWorkflowDto
            {
                NextHandlers = [stepOrg],
                NextStepCode = stepInfo.Key,
                NextStepName = stepInfo.Value,
                Opinion = "办理到一级部门汇总",
                BackToCountersignEnd = false,
                IsSms = false,
                IsForwarded = false,
                HandlerType = EHandlerType.AssignedUser,
                BusinessType = EBusinessType.Department,
                FlowDirection = EFlowDirection.OrgToOrg,
                StepType = EStepType.Summary,
                WorkflowId = order.WorkflowId,
                StepId = stepNextInfo.StepId,
            }
        };
        _orderController.Handle(handleDto).GetAwaiter().GetResult();
        return this;
    }

    public OrderServiceMock 发布工单(Action aciton = null)
    {
        aciton?.Invoke();
        var baseData = _orderController.PublishOrderPageBase(CreateOrderOutDto.Id).GetAwaiter().GetResult();
        var inDto = new PublishOrderDto
        { 
            Id = CreateOrderOutDto.Id,
            PublishState = false,
            ArrangeTitle = baseData.OrderTitle,
            ArrangeContent = baseData.Content,
            ArrangeOpinion = baseData.ActualOpinion,
            IdNames = [baseData.ActualHandleOrgName],
            ProPublishState = false,
            Resolve = true
        };

        _orderController.PublishOrder(inDto).GetAwaiter().GetResult();
        return this;
    }

    public OrderServiceMock 发送回访短信(Action aciton = null)
    {
        aciton?.Invoke();
        var id = _orderVisitRepository.Queryable()
            .Where(m => m.No == CreateOrderOutDto.No)
            .Select(m => m.Id)
            .First();
        var inDto = new VisitSmsInDto
        {
            Ids = [id]
        };
        _orderController.VisitPushSMSAsync(inDto).GetAwaiter().GetResult();
        return this;
    }
}
