﻿using Hotline.Orders;
using Hotline.Share.Enums.Snapshot;
using SqlSugar;
using System.ComponentModel;
using XF.Domain.Repository;

namespace Hotline.Snapshot;

/// <summary>
/// 工单表扩展
/// Id 和 Order.Id 相同
/// </summary>
[Description("工单表扩展")]
public class OrderSnapshot : CreationSoftDeleteEntity
{
    /// <summary>
    /// 行业Id
    /// <inheritdoc cref="Industry"/> 表的Id
    /// </summary>
    [SugarColumn(ColumnDescription = "行业Id")]
    public string IndustryId { get; set; }

    /// <summary>
    /// 行业名称
    /// <inheritdoc cref="Industry"/> 表的Name
    /// </summary>
    [SugarColumn(ColumnDescription = "行业名称")]
    public string IndustryName { get; set; }

    /// <summary>
    /// 社区Id
    /// <inheritdoc cref="CommunityInfo"/> 表的Id
    /// </summary>
    [SugarColumn(ColumnDescription = "社区Id")]
    public string? CommunityId { get; set; }

    /// <summary>
    /// 社区名字
    /// </summary>
    [SugarColumn(ColumnDescription = "社区名字")]
    public string? CommunityName { get; set; }

    /// <summary>
    /// 作业类型
    /// </summary>
    [SugarColumn(ColumnDescription = "作业类型")]
    public string? JobType { get; set; }

    /// <summary>
    /// 经营单位类别
    /// </summary>
    [SugarColumn(ColumnDescription = "经营单位类别")]
    public string? BusinessUnitType { get; set; }

    /// <summary>
    /// 作业场所
    /// </summary>
    [SugarColumn(ColumnDescription = "作业场所")]
    public string? Workplace { get; set; }

    /// <summary>
    /// 场所名称(多个场所使用 - 连接)
    /// </summary>
    [SugarColumn(ColumnDescription = "场所名称")]
    public string? WorkplaceName { get; set; }

    /// <summary>
    /// 作业时间
    /// </summary>
    [SugarColumn(ColumnDescription = "作业时间")]
    public DateTime? StartWorkTime { get; set; }

    /// <summary>
    /// 作业结束时间
    /// </summary>
    [SugarColumn(ColumnDescription = "作业时间")]
    public DateTime? EndWorkTime { get; set; }

    /// <summary>
    /// 小程序公开
    /// </summary>
    public bool? IsAppOpened { get; set; }

    #region 网格员回复

    /// <summary>
    /// 网格员系统交互日志
    /// </summary>
    [SugarColumn(ColumnDescription = "网格员系统交互日志")]
    public string? GuiderAccLog { get; set; }

    /// <summary>
    /// 网格员回复截止日期
    /// </summary>
    [SugarColumn(ColumnDescription = "网格员回复截止日期")]
    public DateTime? DeadLine { get; set; }

    /// <summary>
    /// 网格员是否办理
    /// </summary>
    [SugarColumn(ColumnDescription = "网格员是否办理")]
    public bool? IsDeal { get; set; }

    /// <summary>
    /// 网格E通编号
    /// </summary>
    [SugarColumn(ColumnDescription = "网格E通编号")]
    public string? NetworkENumber { get; set; }

    /// <summary>
    /// 是否属实
    /// </summary>
    [SugarColumn(ColumnDescription = "是否属实")]
    public bool? IsTruth { get; set; }

    /// <summary>
    /// 是否重复
    /// </summary>
    [SugarColumn(ColumnDescription = "是否重复")]
    public bool? IsRepetition { get; set; }

    /// <summary>
    /// 是否隐患
    /// </summary>
    [SugarColumn(ColumnDescription = "是否隐患")]
    public bool? IsDanger { get; set; }

    /// <summary>
    /// 网格员回复内容
    /// </summary>
    [SugarColumn(ColumnDescription = "网格员回复内容", Length = 2048)]
    public string? NetworkRemark { get; set; }

    /// <summary>
    /// 网格员办理时间
    /// </summary>
    [SugarColumn(ColumnDescription = "网格员办理时间")]
    public DateTime? ReplyDate { get; set; }

    /// <summary>
    /// 网格员姓名
    /// </summary>
    [SugarColumn(ColumnDescription = "网格员姓名")]
    public string? MemberName { get; set; }

    /// <summary>
    /// 网格员电话
    /// </summary>
    [SugarColumn(ColumnDescription = "网格员电话")]
    public string? MemberMobile { get; set; }

    /// <summary>
    /// 办理人账号
    /// </summary>
    [SugarColumn(ColumnDescription = "办理人账号")]
    public string? ReplyUserName { get; set; }

    /// <summary>
    /// 办理部门
    /// </summary>
    [SugarColumn(ColumnDescription = "办理部门")]
    public string? ReplyBMName { get; set; }

    /// <summary>
    /// 办理状态 1，7：流转 2：办结 3：退回 4：网格员签收 5：消息推送 6: 超时自动退单
    /// </summary>
    [SugarColumn(ColumnDescription ="网格员办理状态")]
    public EGuiderSystemReplyType? ReplyResultType { get; set; }
    #endregion
}
